[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# FactSet Private Markets client library for Python

[![API Version](https://img.shields.io/badge/api-v1.2.0-blue)]()
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.FactSetPrivateMarkets)](https://pypi.org/project/fds.sdk.FactSetPrivateMarkets/)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

FactSet Private Markets API encompasses Private Company Financials data to start. Private Company firmographics can be found in the FactSet Entity API. The future plans of this service will include additional Private Markets data, such as Private Equity/Venture Capital.<p><b>Rate limit is set to 10 requests per second</b>.</p>


This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 1.2.0
- SDK version: 1.0.13
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.FactSetPrivateMarkets==1.0.13
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.FactSetPrivateMarkets==1.0.13
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.FactSetPrivateMarkets
from fds.sdk.FactSetPrivateMarkets.api import data_items_api
from fds.sdk.FactSetPrivateMarkets.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.FactSetPrivateMarkets.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.FactSetPrivateMarkets.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.FactSetPrivateMarkets.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = data_items_api.DataItemsApi(api_client)
    category = "FINANCIALS" # str | Filters the list of available fields per endpoint  -   * **FINANCIALS** = Financials endpoint data items, such as Assets, Liabilities, and EBITDA.   * **NON_PERIODIC** = Non-Periodic endpoint data items, such as ISO Currency, Current CEO, and Investor List.  (optional)

    try:
        # Available private company fields, metrics, and ratios.
        # example passing only required values which don't have defaults set
        # and optional values
        api_response = api_instance.get_fields(category=category)

        pprint(api_response)
    except fds.sdk.FactSetPrivateMarkets.ApiException as e:
        print("Exception when calling DataItemsApi->get_fields: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Available private company fields, metrics, and ratios.
    #     api_response, http_status_code, response_headers = api_instance.get_fields_with_http_info(category=category)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetPrivateMarkets.ApiException as e:
    #     print("Exception when calling DataItemsApi->get_fields: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Available private company fields, metrics, and ratios.
    #     async_result = api_instance.get_fields_async(category=category)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.FactSetPrivateMarkets.ApiException as e:
    #     print("Exception when calling DataItemsApi->get_fields: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Available private company fields, metrics, and ratios.
    #     async_result = api_instance.get_fields_with_http_info_async(category=category)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetPrivateMarkets.ApiException as e:
    #     print("Exception when calling DataItemsApi->get_fields: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.FactSetPrivateMarkets

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.FactSetPrivateMarkets.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.FactSetPrivateMarkets

configuration = fds.sdk.FactSetPrivateMarkets.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.FactSetPrivateMarkets

configuration = fds.sdk.FactSetPrivateMarkets.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.FactSetPrivateMarkets

configuration = fds.sdk.FactSetPrivateMarkets.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/content*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*DataItemsApi* | [**get_fields**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/DataItemsApi.md#get_fields) | **GET** /factset-private-markets/v1/fields | Available private company fields, metrics, and ratios.
*FinancialsApi* | [**get_financials**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/FinancialsApi.md#get_financials) | **GET** /factset-private-markets/v1/financials | Returns Private Company Financials for a list of ids.
*FinancialsApi* | [**get_financials_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/FinancialsApi.md#get_financials_for_list) | **POST** /factset-private-markets/v1/financials | Returns Private Company Financials for a list of ids.
*NonPeriodicApi* | [**get_non_periodic**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/NonPeriodicApi.md#get_non_periodic) | **GET** /factset-private-markets/v1/non-periodic | Returns Private Company reference data for a list of ids.
*NonPeriodicApi* | [**get_non_periodic_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/NonPeriodicApi.md#get_non_periodic_for_list) | **POST** /factset-private-markets/v1/non-periodic | Returns Private Company reference data for a list of ids.
*UniverseApi* | [**get_universe**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/UniverseApi.md#get_universe) | **GET** /factset-private-markets/v1/universe | Get Entity Candidates and Matches for a single name and attributes.
*UniverseApi* | [**get_universe_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/UniverseApi.md#get_universe_for_list) | **POST** /factset-private-markets/v1/universe | Get a list of Entity Candidates and Matches for a requested list of up to 25 names and attributes.


## Documentation For Models

 - [EntityMatch](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/EntityMatch.md)
 - [EntityMatchRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/EntityMatchRequest.md)
 - [EntityMatchRequestInput](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/EntityMatchRequestInput.md)
 - [EntityMatchesResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/EntityMatchesResponse.md)
 - [ErrorResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/ErrorResponse.md)
 - [ErrorResponseSubErrors](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/ErrorResponseSubErrors.md)
 - [Field](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/Field.md)
 - [FieldsResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/FieldsResponse.md)
 - [Financials](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/Financials.md)
 - [FinancialsFields](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/FinancialsFields.md)
 - [FinancialsRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/FinancialsRequest.md)
 - [FinancialsResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/FinancialsResponse.md)
 - [NonPeriodic](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/NonPeriodic.md)
 - [NonPeriodicFields](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/NonPeriodicFields.md)
 - [NonPeriodicRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/NonPeriodicRequest.md)
 - [NonPeriodicResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/NonPeriodicResponse.md)
 - [PrivateMarketIds](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetPrivateMarkets/v1/docs/PrivateMarketIds.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.FactSetPrivateMarkets.apis and fds.sdk.FactSetPrivateMarkets.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.FactSetPrivateMarkets.api.default_api import DefaultApi`
- `from fds.sdk.FactSetPrivateMarkets.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.FactSetPrivateMarkets
from fds.sdk.FactSetPrivateMarkets.apis import *
from fds.sdk.FactSetPrivateMarkets.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

